
import os
import sys
import dbus

arg_lang = ""

################################
# 常量

# for get_env_lang()
STR_GET_ENV_LANG_ZH     = "语言环境为中文"
STR_GET_ENV_LANG_EN     = "语言环境为英文"
STR_GET_ENV_LANG_UNKNOW = "语言环境未知"

# for is_root()
STR_IS_ROOT_TRUE  = "当前用户为root权限"
STR_IS_ROOT_FALSE = "当前用户没有root权限"

# mode of firewall
KSC_FIREWALL_ALL_MODE   = 0
KSC_FIREWALL_PUBLIC     = 1
KSC_FIREWALL_PRIVATE    = 2
KSC_FIREWALL_OFF        = 3
KSC_FIREWALL_TAKE_OVER  = 4


################################
# 环境检查函数

def get_env_lang():
#    lang = os.getenv("LANG")
#    if lang.startswith("zh"):
#        return STR_GET_ENV_LANG_ZH
#    elif lang.startswith("en"):
#        return STR_GET_ENV_LANG_EN
#    else:
#        return STR_GET_ENV_LANG_UNKNOW
#
    if arg_lang == "zh":
        return STR_GET_ENV_LANG_ZH
    elif arg_lang == "en":
        return STR_GET_ENV_LANG_EN
    else:
        return STR_GET_ENV_LANG_UNKNOW

def is_root():
    if os.geteuid() == 0:
        print(STR_IS_ROOT_TRUE)
        return True
    else:
        print(STR_IS_ROOT_FALSE)
        return False

################################
# 辅助函数
def l_print(zh_str, en_str) :
    if STR_GET_ENV_LANG_ZH == get_env_lang() :
        print(zh_str);
    else :
        print(en_str);


################################
# 功能函数
def check_firewall_log():
    mode = 0
    try:
        bus = dbus.SystemBus()
        if not bus.get_name_owner("com.ksc.defender"):
            raise dbus.exceptions.DubsException("com.ksc.defender 服务没开启")
    except dbus.exceptions.DBusException as e:
        l_print("[WARNING] 连接dbus服务失败:" + str(e),
                "[WARNING] Failed to connect to D-Bus service:" + str(e))
        exit(1)

    try:
        remote_obj = bus.get_object("com.ksc.defender", "/firewall")
        iface = dbus.Interface(remote_obj, "com.ksc.defender.firewall")
        mode = iface.get_firewall_mode()
    except dbus.exceptions.DBusException as e:
        l_print("[WARNING] 连接dbus服务失败:" + str(e),
                "[WARNING] Failed to connect to D-Bus service:" + str(e))
        exit(1)


    if not mode==KSC_FIREWALL_OFF:
        l_print("[OK] 防火墙日志已启用",
                "[OK] Firewall logger is enable")
    else:
        l_print("[WARNING] 防火墙日志未启用",
                "[WARNING] Firewall logger is disable")

################################
# main
if __name__ == "__main__":
    arg_lang = sys.argv[1]

    if False == is_root() :
        l_print("[WARNING] 非root权限，此程序需要root权限",
                "[WARNING] Non root permission, this program requires root permission")
        exit(1)

    check_firewall_log()
    exit(0)
